<?php
require_once 'RentalAction.php';
require_once 'Debugger.php';
require_once 'Logger.php';

class RentalCompany {

    protected $fleet = array();
    protected $rentalActions = array();
    protected $debugger;
    protected $logger;

    /**
     * @Inject
     */
    public function __construct(Debugger $debugger) {
        $this->debugger = $debugger;
    }

    /**
     * @Inject(optional=true)
     */
    public function setLogger(Logger $logger) {
        $this->logger = $logger;
    }

    public function addToFleet($id, Vehicle $vehicle) {
        $this->fleet[$id]  = $vehicle;
        $this->debug("Neues Auto im Fuhrpark: " . $vehicle->getManufacturer());
    }

    public function rentVehicle(Vehicle $vehicle, Customer $customer) {
        $vehicleId = array_search($vehicle, $this->fleet);
        if ($vehicleId === false) {
        	throw new UnknownVehicleException();
        }
        if (!$this->isVehicleAvailable($vehicle)) {
        	throw new VehicleNotAvailableException();
        }
        $rentalAction = new RentalAction($vehicle, $customer);
        $this->rentalActions[] = $rentalAction;
        $this->debug("Neuer Mietvorgang: " . $customer->getName() . " leiht " . $vehicle->getManufacturer());

        return $rentalAction;
    }

    public function returnVehicle(Vehicle $vehicle) {
        foreach ($this->rentalActions as $rentalAction) {
        	if ($rentalAction->getVehicle() !== $vehicle) {
        		continue;
        	}
        	if ($rentalAction->isReturned()) {
        		continue;
        	}
        	$rentalAction->markVehicleReturned();
            $this->debug("Rckgabe: " . $rentalAction->getCustomer()->getName() . " gibt " . $vehicle->getManufacturer() . " zurck.");
        	return true;
        }
        return false;
    }

    public function isVehicleAvailable(Vehicle $vehicle) {
        foreach ($this->rentalActions as $rentalAction) {
        	if ($rentalAction->getVehicle() !== $vehicle) {
        		continue;
        	}
        	if ($rentalAction->isReturned()) {
        		continue;
        	}
        	return false;
        }
        return true;
    }

    protected function debug($message) {
        $this->debugger->debug($message);
    }
}
?>